@@ -29,7 +29,7 @@ import _location
 log = logging.getLogger("location.export")
 log.propagate = 1
 
-flags = {
+FLAGS = {
 	_location.NETWORK_FLAG_ANONYMOUS_PROXY    : "A1",
 	_location.NETWORK_FLAG_SATELLITE_PROVIDER : "A2",
 	_location.NETWORK_FLAG_ANYCAST            : "A3",
@@ -39,11 +39,8 @@ class OutputWriter(object):
 	suffix = "networks"
 	mode = "w"
 
-	def __init__(self, f, prefix=None, flatten=True):
-		self.f, self.prefix, self.flatten = f, prefix, flatten
-
-		# The previously written network
-		self._last_network = None
+	def __init__(self, f, prefix=None):
+		self.f, self.prefix = f, prefix
 
 		# Immediately write the header
 		self._write_header()
@@ -60,18 +57,6 @@ class OutputWriter(object):
 	def __repr__(self):
 		return "<%s f=%s>" % (self.__class__.__name__, self.f)
 
-	def _flatten(self, network):
-		"""
-			Checks if the given network needs to be written to file,
-			or if it is a subnet of the previously written network.
-		"""
-		if self._last_network and network.is_subnet_of(self._last_network):
-			return True
-
-		# Remember this network for the next call
-		self._last_network = network
-		return False
-
 	def _write_header(self):
 		"""
 			The header of the file
@@ -84,16 +69,8 @@ class OutputWriter(object):
 		"""
 		pass
 
-	def _write_network(self, network):
-		self.f.write("%s\n" % network)
-
 	def write(self, network):
-		if self.flatten and self._flatten(network):
-			log.debug("Skipping writing network %s" % network)
-			return
-
-		# Write the network to file
-		self._write_network(network)
+		self.f.write("%s\n" % network)
 
 	def finish(self):
 		"""
@@ -114,7 +91,7 @@ class IpsetOutputWriter(OutputWriter):
 	def _write_header(self):
 		self.f.write("create %s hash:net family inet hashsize 1024 maxelem 65536\n" % self.prefix)
 
-	def _write_network(self, network):
+	def write(self, network):
 		self.f.write("add %s %s\n" % (self.prefix, network))
 
 
@@ -130,7 +107,7 @@ class NftablesOutputWriter(OutputWriter):
 	def _write_footer(self):
 		self.f.write("}\n")
 
-	def _write_network(self, network):
+	def write(self, network):
 		self.f.write("	%s,\n" % network)
 
 
@@ -142,14 +119,9 @@ class XTGeoIPOutputWriter(OutputWriter):
 	suffix = "iv"
 	mode = "wb"
 
-	def _write_network(self, network):
-		for address in (network.first_address, network.last_address):
-			# Convert this into a string of bits
-			bytes = socket.inet_pton(
-				network.family, address,
-			)
-
-			self.f.write(bytes)
+	def write(self, network):
+		self.f.write(network._first_address)
+		self.f.write(network._last_address)
 
 
 formats = {
@@ -185,8 +157,14 @@ class Exporter(object):
 
 				writers[asn] = self.writer.open(filename, prefix="AS%s" % asn)
 
+			# Filter countries from special country codes
+			country_codes = [
+				country_code for country_code in countries if not country_code in FLAGS.values()
+			]
+
 			# Get all networks that match the family
-			networks = self.db.search_networks(family=family)
+			networks = self.db.search_networks(family=family,
+				country_codes=country_codes, asns=asns, flatten=True)
 
 			# Walk through all networks
 			for network in networks:
@@ -203,10 +181,10 @@ class Exporter(object):
 					pass
 
 				# Handle flags
-				for flag in flags:
+				for flag in FLAGS:
 					if network.has_flag(flag):
 						# Fetch the "fake" country code
-						country = flags[flag]
+						country = FLAGS[flag]
 
 						try:
 							writers[country].write(network)
